#include "jlcxx/functions.hpp"

#include "Geant4Wrap.h"
#include "G4StateManager.hh"
#include "G4ios.hh"
#include "G4PVPlacement.hh"
#include "G4ParticleTable.hh"

#include <stdexcept>

// --------------------------------------------------------------------------
G4bool G4JLExceptionHandler::Notify(const char* originOfException,
                                    const char* exceptionCode,
                                    G4ExceptionSeverity severity,
                                    const char* description) {
  std::ostringstream message;
  message << "*** G4Exception : " << exceptionCode << G4endl
          << "      issued by : " << originOfException << G4endl
          << description << G4endl;

  switch(severity) {
  case FatalException:
    throw std::runtime_error( message.str() );
    break;
  case FatalErrorInArgument:
    throw std::invalid_argument( message.str() );
    break;
  case RunMustBeAborted:
    throw std::runtime_error( message.str() );
    break;
  case EventMustBeAborted:
    throw std::runtime_error( message.str() );
    break;
  default:
    throw std::runtime_error( message.str() );
    break;
  }
  return false;
}
void SetParticleByName(G4ParticleGun* gun, const char* pname)
{
  G4ParticleTable* particleTable= G4ParticleTable::GetParticleTable();
  G4ParticleDefinition* pd= particleTable-> FindParticle(pname);
  if (pd != 0) {
    gun-> SetParticleDefinition(pd);
  } else {
    G4cout << "*** \"" << pname << "\" is not registered "
	   << "in available particle list" << G4endl;
  }
}
G4ParticleDefinition* FindParticle(const char* pname)
{
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  return particleTable-> FindParticle(pname);
}

char* G4JL_getenv(const char* x) { return std::getenv(x); };

int G4JL_setenv(const char* x, const char* v) { return setenv(x, v, 1); }

void G4JL_init(void) {
  auto sm = G4StateManager::GetStateManager();
  sm-> SetExceptionHandler(new G4JLExceptionHandler());
}
